#include <cstring>
#include <sstream>
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <algorithm>
#include <vector>
#include <cmath>
#include "LIBSA.h"
#include "death.h"
#include "functions.h"
#include "print_help.h"
#include "receptor_parser.cpp"
#include "ligand_parser.cpp"
#include "affinity_filter.cpp"
#include "calculate_contacts.cpp"
#include "histogram_maker.cpp"
#include "highpass_filter.cpp"
#include "calculate_snr.cpp"

int main (int argc, char **argv) {

bool verbose = 0;

if ( argc != 5 ) {
	std::cout << "Function usage ..." << std::endl;
	std::cout << "./LIBSA -ligand <ligand.pdb> -receptor <receptor.pdb>" << std::endl;
	print_death();
	exit(0);
}

std::string a, b;
a = std::string(argv[1]);
b = std::string(argv[3]);

if ( a.compare("-ligand") != 0 && a.compare("-receptor") != 0 ) {
	std::cout << "Unknown option: " << a << std::endl;
	exit(0);
}

if ( b.compare("-ligand") != 0 && b.compare("-receptor") != 0 ) {
	std::cout << "Unknown option: " << b << std::endl;
	exit(0);
}

//Set nice variables for the ligand and receptor arguments
unsigned int ligand_arg=0, receptor_arg=0;
if ( a.compare("-ligand") == 0 ) {
	ligand_arg = 2;
	receptor_arg = 4;
} else {
	ligand_arg = 4;
	receptor_arg = 2;
}

//Rename the command line inputs to something easier to remember
char *ligand_file = (char*)malloc(sizeof(char)*256);
std::cout << "Ligand file is " << argv[ligand_arg] << std::endl;
std::strcpy(ligand_file, argv[ligand_arg]);
char *receptor_file = (char*)malloc(sizeof(char)*256);
std::cout << "Receptor file is " << argv[receptor_arg] << std::endl;
std::strcpy(receptor_file, argv[receptor_arg]);

//Check to see if the ligand file exists
FILE *pFile;
pFile = fopen (ligand_file, "r");
if (pFile == NULL) 
{
  cout << "Error opening the ligand file: " << ligand_file << endl;
  exit(0);
}
fclose(pFile); //reopen file later when we parse it

//Check to see if the receptor file exists
FILE *pFile2;
pFile2 = fopen (receptor_file, "r");
if (pFile2 == NULL) 
{
  cout << "Error opening the receptor file: " << receptor_file << endl;
  exit(0);
}
fclose(pFile2); //reopen file later when we parse it

//Parse up the receptor file
LIBSA::Receptor *receptor_root = new LIBSA::Receptor;
receptor_root->next = NULL;
LIBSA::Receptor **pprec = NULL;
pprec = &receptor_root;
receptor_root->add_node(receptor_root, pprec, receptor_file);
receptor_root->delete_head_node(receptor_root,pprec); //get rid of empty head node
receptor_root->print_receptor_list_summary(receptor_root);

//Parse up the ligand file
LIBSA::Ligand *ligand_root = new LIBSA::Ligand;
ligand_root->next = NULL;
LIBSA::Ligand **pplig = NULL;
pplig = &ligand_root;
ligand_root->parse_ligand(ligand_root, pplig, ligand_file);
ligand_root->delete_head_node(ligand_root, pplig);
ligand_root->print_ligand_list_summary(ligand_root);

//Make conductor variables for searching through the linked lists
LIBSA::Ligand *ligand_conductor = ligand_root;
std::vector<float> affinity;
while ( ligand_conductor != NULL ) {
	affinity.push_back(ligand_conductor->affinity);
	ligand_conductor = ligand_conductor->next;
}

bool do_affinity_filter = 0;
if ( do_affinity_filter ) {
	std::vector<unsigned int> nodes_to_remove;
	ligand_root->affinity_filter(affinity, nodes_to_remove);
	//Delete the nodes from biggest to smallest
	std::cout << "REMARK:DELETE_" << nodes_to_remove.size() << "_NODES_FROM_THE_LINKED_LIST" << std::endl;
	for (int i=nodes_to_remove.size()-1;i>=0;i--) {
		if(verbose)std::cout << "Delete node #" << nodes_to_remove.at(i) << std::endl;
		int which = nodes_to_remove.at(i);
		ligand_root->delete_node(ligand_root, pplig, which);
	}
}

//Now, go through the receptors and the ligand poses and calculate the "contact spectrum"
LIBSA::Receptor *receptor_conductor = receptor_root;
ligand_conductor = ligand_root;
std::vector<unsigned int> raw_contacts;
while ( receptor_conductor != NULL ) {
	ligand_conductor = ligand_root;
	while ( ligand_conductor != NULL ) {
		for (unsigned int i=0;i<receptor_conductor->x.size();i++)
			for (unsigned int j=0;j<ligand_conductor->x.size();j++)
			{	
				bool isContact = calculate_contacts(receptor_conductor->x.at(i),
													receptor_conductor->y.at(i),
													receptor_conductor->z.at(i),
													ligand_conductor->x.at(j),
													ligand_conductor->y.at(j),
													ligand_conductor->z.at(j),
													receptor_conductor->element.at(i) );
				if ( isContact ) {
					raw_contacts.push_back(receptor_conductor->residueIndex.at(i));
				}
			}
		ligand_conductor = ligand_conductor->next;
	}
	receptor_conductor = receptor_conductor->next;
}

std::vector<float> histogram_residue_intensity;
std::vector<int> histogram_residue_index;
int max = 1;
receptor_root->findMaxResID(receptor_root, &max);
std::cout << "REMARK:MAX_RESID " << max << std::endl;
make_histogram(raw_contacts, histogram_residue_index, histogram_residue_intensity, max);

bool do_fourier_filter = 0;
if (do_fourier_filter)
	butterworth_hpf(histogram_residue_intensity);

for (unsigned int i=0;i<histogram_residue_intensity.size();i++)
	std::cout 	<< "MAIN:HIST " 
				<< histogram_residue_index.at(i) 
				<< " " 
				<< histogram_residue_intensity.at(i) 
				<< std::endl;

//determine which residues are noise and which are signal
std::vector<bool> isSignal;
for (unsigned int i=0;i<histogram_residue_index.size();i++)
	isSignal.push_back(0);

receptor_conductor = receptor_root;
while ( receptor_conductor != NULL ) {
	for (unsigned int i=0;i<receptor_conductor->residueIndex.size();i++)
		if ( receptor_conductor->beta.at(i) > 0 ) {
			isSignal.at( receptor_conductor->residueIndex.at(i) ) = 1;
			if(verbose) std::cout << "Signal at residue " << receptor_conductor->residueIndex.at(i) << std::endl;
		}
	receptor_conductor = receptor_conductor->next;
}

for (unsigned int i=0;i<isSignal.size();i++)
	if(verbose)std::cout 		
				<< "Signal Residue Found @ " << histogram_residue_index.at(i)
				<< " beta-value-bool " << isSignal.at(i) 
				<< " histogram value " << histogram_residue_intensity.at(i)
				<< std::endl;

float SNR = 0.0;
calculate_snr( histogram_residue_intensity, isSignal, &SNR );
std::cout << "REMARK:SNR " << SNR << std::endl;

std::cout << "REMARK:EXITING_MAIN_NORMALLY" << std::endl;
return 0;
}


