#include "LIBSA.h"

bool debug = 0;

void LIBSA::Receptor::findMaxResID (LIBSA::Receptor *root, int *max)
{
	LIBSA::Receptor *conductor = root;
	*max = 1;
	while ( conductor != NULL ) {
		for ( unsigned int i=0;i<conductor->residueIndex.size();i++ )
			if ( conductor->residueIndex.at(i) > *max )
				*max = conductor->residueIndex.at(i);
		conductor = conductor->next;
	}
	std::cout << "REMARK:FOUND_MAX_RESID: " << *max << std::endl;
	return;
}

bool LIBSA::Receptor::isAtomRecord (std::string line) 
{
	char waste1[256];
	char waste2[256];
	sscanf(line.c_str(), "%s %s", waste1, waste2);
	std::string temp = string(waste1);
	temp = remove_spaces_from_string(temp);
	if ( temp.compare("ATOM") )
		return 0;
	else
		return 1;
}

void LIBSA::Receptor::delete_head_node(LIBSA::Receptor *root, LIBSA::Receptor **pp) 
{
	LIBSA::Receptor *temp;
	temp = root->next;
	*pp = temp;
	free(root);
	return;
}

void LIBSA::Receptor::delete_node(LIBSA::Receptor *root, LIBSA::Receptor **pp, unsigned int which_node) 
{
     if ( which_node == 1 ) {
          delete_head_node(root, pp);
          return;
     }

     LIBSA::Receptor *conductor = root;
     for (unsigned int i = 2;i<which_node;i++) 
        conductor = conductor->next;


     LIBSA::Receptor *bad = conductor->next;
     conductor->next = conductor->next->next;
     free(bad);

     return;
}

void LIBSA::Receptor::print_receptor_list_summary(LIBSA::Receptor *root) 
{
	LIBSA::Receptor *conductor = root;
	unsigned int numFiles = 1;
	while ( conductor != NULL ) {
		std::cout << "Receptor File #" << numFiles << " ...number of atoms in the receptor is " << conductor->x.size() << std::endl;
		/*for (unsigned int i=0;i<conductor->x.size();i++)
			std::cout << "Print some stuff " 	<< conductor->x.at(i) 
												<< " " 
												<< conductor->y.at(i) 
												<< " " 
												<< conductor->z.at(i) 
												<< std::endl;*/
		conductor = conductor->next;
		numFiles++;
	}
}

void LIBSA::Receptor::add_node (LIBSA::Receptor *root, LIBSA::Receptor **pp, char *file)
{
	//Each node object holds a full receptor structure
	//Multiple receptor nodes are required to store the sturctures of multiple receptors
	LIBSA::Receptor *node = new LIBSA::Receptor;
	LIBSA::Receptor *conductor = root;
	node->next = NULL;

	//If the root node does not exist set it to the new node
	if ( conductor == NULL ) {
		root = node;
		pp = &node;
	} else {
		while (conductor->next != NULL)
			conductor = conductor->next;
		conductor->next = node;
	}

	ifstream myfile ( file );
	std::string temp;
	if (myfile) {
		char header[6];
		unsigned int atomIndex;
		char atomName[4];
		char residue[3];
		char chain[1];
		unsigned int residueIndex;
		float x, y, z, alpha, beta;
		char segment[2];
		char element[1];

		unsigned int num_lines = 0;
		while ( getline(myfile,temp) ) {
			if ( isAtomRecord(temp) ) {
				num_lines++;
				sscanf (temp.c_str(),"%s %d %s %s %s %d %f %f %f %f %f %s %s",header, &atomIndex, atomName, residue, chain, &residueIndex, &x, &y, &z, &alpha, &beta, segment, element);
				node->header.push_back(header);
				node->atomIndex.push_back(atomIndex);
				node->atomName.push_back(atomName);
				node->residue.push_back(residue);
				node->chain.push_back(chain);
				node->residueIndex.push_back(residueIndex);
				node->x.push_back(x);
				node->y.push_back(y);
				node->z.push_back(z);
				node->alpha.push_back(alpha);
				node->beta.push_back(beta);
				node->element.push_back(element);
				if(debug) std::cout 	<< "Validation: header & atomIndex " 
							<< node->header.at(num_lines-1) 
							<< " " 
							<< node->atomIndex.at(num_lines-1)
							<< " " 
							<< node->atomName.at(num_lines-1)
							<< " "
							<< node->element.at(num_lines-1)
							<< " "
							<< node->alpha.at(num_lines-1)
							<< " "
							<< node->x.at(num_lines-1)
							<< std::endl;
			}

		}
	if(debug) std::cout << "There are " << num_lines << " lines in the file" << std::endl;
	}
	myfile.close();
	return;
}

bool LIBSA::Receptor::parse_receptor(LIBSA::Receptor *root, LIBSA::Receptor **pp, char *file) 
{
/*Parse a PDB file that looks like this (PDB exerts from 3SN6)
REMARK original generated coordinate pdb file
ATOM      1  N   GLU R  30      26.833  30.580 -20.526  1.00  0.00      U    N
ATOM      2  HT1 GLU R  30      26.399  30.373 -21.403  0.00  0.00      U    H
ATOM      3  HT2 GLU R  30      27.046  29.731 -20.042  0.00  0.00      U    H
...
ATOM   1376 HG11 VAL R 117      19.096   5.417  -4.337  0.00  0.00      U    H
ATOM   1377 HG12 VAL R 117      19.194   4.515  -2.795  0.00  0.00      U    H
ATOM   1378 HG13 VAL R 117      19.541   6.286  -2.828  0.00  0.00      U    H
ATOM   1379  CG2 VAL R 117      21.714   6.116  -4.583  1.00  0.00      U    C
ATOM   1380 HG21 VAL R 117      21.121   6.195  -5.519  0.00  0.00      U    H
ATOM   1381 HG22 VAL R 117      21.712   7.109  -4.083  0.00  0.00      U    H
ATOM   1382 HG23 VAL R 117      22.758   5.871  -4.868  0.00  0.00      U    H
...
ATOM   4624  HB1 CYS R 341      40.252  25.136  15.639  0.00  0.00      U    H
ATOM   4625  HB2 CYS R 341      41.520  25.259  16.879  0.00  0.00      U    H
ATOM   4626  SG  CYS R 341      41.837  23.349  15.399  1.00  0.00      U    S
ATOM   4627  HG1 CYS R 341      42.473  24.237  14.633  0.00  0.00      U    H
     http://deposit.rcsb.org/adit/docs/pdb_atom_format.html
     1-6 	= Record name
     7-11 	= Atom Serial Number
     13-16 	= Atom Name
     17 	= optional
     18-20	= Residue Name
     22 	= Chain Identifier
     23-26 	= residueIndex
     27 	= Code for insertion of residues
     31-38 	= X-coordinate
     39-46	= Y-coordinate
     47-54	= Z-coordiante
     55-60 	= Occupancy
     61-66	= Temperature Factor
     73-76 	= Segment Identifier (left justified)
     77-78 	= Element Symbol
     79-80	= Charge on the atom
*/
     /*std::cout << "root is " << root << " in the before" << std::endl;
	 std::cout << "pp is " << pp << " in the before" << std::endl;
	 std::cout << "*pp is " << *pp << " in the before" << std::endl;*/
     add_node(root, pp, file);
     /*std::cout << "The next file is " << root->next << std::endl;
     std::cout << "root is " << root << " in the after" << std::endl;
	 std::cout << "pp is " << pp << " in the after" << std::endl;
	 std::cout << "*pp is " << *pp << " in the after" << std::endl;*/
     print_receptor_list_summary(root);
     // have functions to check the integrity of the linked list ... check_linked_list_validity(root);
     return 1;

}